(function() {

//-- Backbone Objects --------------------------------------------------//
    
    //-- Models --------------------------------------------------------//
    
        var Article = Backbone.Model;
    
    
    //-- Collections ---------------------------------------------------//
    
        var Articles = Backbone.Collection.extend({ 
            model: Article 
        });
        
        
    //-- Views ---------------------------------------------------------//

        var Post = Backbone.View.extend({
            
            tagName: 'article',

            template: '<h2><a href="/#/articles/<%= _id %>"><%= title %></a></h2><p><%= content %></p>',

            events: {
                'dblclick' : 'destroy'
            },

            destroy: function() {

                this.model.destroy();
                
                $(this.el).fadeOut(400, function() {
                    $(this).remove();
                });
            },

            render: function() {

                this.model || $.error("ERROR: No model could be found.");

                $(this.el).append(_.template(this.template, this.model.toJSON()) );

                return this;
            }
            
        });


        var Posts = Backbone.View.extend({
            
            el: $("#content"),

            initialize: function() {
                           
                var that = this.el;
                     
                this.collection.bind('add', function(model) {
                    var post = new Post({ model: model }).render().el;

                    $(that).append(post);
                });

            },

            reset: function() {
                $(this.el).html("");
            }
            
        });
        
    
    //-- Create instances ----------------------------------------------//
    
        var articles = new Articles();
        
        var posts = new Posts({
            collection: articles
        });
        
        
    //-- Router --------------------------------------------------------//
    
    var Router = Backbone.Router.extend({
        
        routes: {
            '/articles/:id'  : 'showArticle'
        },
        
        showArticle: function(id) {
            
            posts.reset();

            var model = articles.get(id);

            this.view = new Post({ model: model }).render().el;

            $("#content").append(this.view);
        }
        
    });
    
    var router = new Router();
        
    Backbone.history.start();
  
//-- Socket.IO ---------------------------------------------------------//
    
    var socket = io.connect('http://localhost');                      
    
    // Receive Article Data
    socket.on('updateDB', function(data) {
        articles.add(data);
    });
        
})();